Class: Decidim::Proposals::Admin::CreateProposal
- Inherits:
-
Command
- Object
- Command
- Decidim::Proposals::Admin::CreateProposal
- Includes:
- MultipleAttachmentsMethods
- Defined in:
- app/commands/decidim/proposals/admin/create_proposal.rb
Overview
A command with all the business logic when a user creates a new proposal.
Instance Method Summary collapse
-
#call ⇒ Object
Executes the command.
-
#initialize(form) ⇒ CreateProposal
constructor
Public: Initializes the command.
Constructor Details
#initialize(form) ⇒ CreateProposal
Public: Initializes the command.
form - A form object with the params.
13 14 15 |
# File 'app/commands/decidim/proposals/admin/create_proposal.rb', line 13 def initialize(form) @form = form end |
Instance Method Details
#call ⇒ Object
Executes the command. Broadcasts these events:
-
:ok when everything is valid, together with the proposal.
-
:invalid if the form was not valid and we could not proceed.
Returns nothing.
23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 |
# File 'app/commands/decidim/proposals/admin/create_proposal.rb', line 23 def call return broadcast(:invalid) if form.invalid? if return broadcast(:invalid) if end transaction do create_proposal (first_weight: ) if if form.created_in_meeting? end send_notification broadcast(:ok, proposal) end |