Class: Decidim::Proposals::Admin::ParticipatoryTextProposalForm

Inherits:
Admin::ProposalBaseForm
  • Object
show all
Defined in:
app/forms/decidim/proposals/admin/participatory_text_proposal_form.rb

Overview

A form object to be used when admin users want to create a proposal through the participatory texts.

Instance Method Summary collapse

Instance Method Details

#map_model(model) ⇒ Object



13
14
15
16
# File 'app/forms/decidim/proposals/admin/participatory_text_proposal_form.rb', line 13

def map_model(model)
  self.title = translated_attribute(model.title)
  self.body = translated_attribute(model.body)
end