Class: Decidim::Proposals::Admin::ProposalsImportForm
- Inherits:
-
Form
- Object
- Form
- Decidim::Proposals::Admin::ProposalsImportForm
- Includes:
- TranslatableAttributes
- Defined in:
- app/forms/decidim/proposals/admin/proposals_import_form.rb
Overview
A form object to be used when admin users want to import a collection of proposals from another component.
Instance Method Summary collapse
- #origin_component ⇒ Object
- #origin_components ⇒ Object
- #origin_components_collection ⇒ Object
- #states ⇒ Object
- #states_collection ⇒ Object
Instance Method Details
#origin_component ⇒ Object
33 34 35 |
# File 'app/forms/decidim/proposals/admin/proposals_import_form.rb', line 33 def origin_component @origin_component ||= origin_components.find_by(id: origin_component_id) end |
#origin_components ⇒ Object
37 38 39 |
# File 'app/forms/decidim/proposals/admin/proposals_import_form.rb', line 37 def origin_components @origin_components ||= current_participatory_space.components.where.not(id: current_component.id).where(manifest_name: :proposals) end |
#origin_components_collection ⇒ Object
41 42 43 44 45 |
# File 'app/forms/decidim/proposals/admin/proposals_import_form.rb', line 41 def origin_components_collection origin_components.map do |component| [component.name[I18n.locale.to_s], component.id] end end |
#states ⇒ Object
29 30 31 |
# File 'app/forms/decidim/proposals/admin/proposals_import_form.rb', line 29 def states super.compact_blank end |
#states_collection ⇒ Object
22 23 24 25 26 27 |
# File 'app/forms/decidim/proposals/admin/proposals_import_form.rb', line 22 def states_collection @states_collection ||= ProposalState.where(component: current_component) + [ProposalState.new(token: "not_answered", title: I18n.t( :not_answered, scope: "decidim.proposals.answers" ))] end |