Class: Decidim::Proposals::Admin::UpdateProposal
- Inherits:
-
Command
- Object
- Command
- Decidim::Proposals::Admin::UpdateProposal
- Includes:
- MultipleAttachmentsMethods
- Defined in:
- app/commands/decidim/proposals/admin/update_proposal.rb
Overview
A command with all the business logic when a user updates a proposal.
Instance Method Summary collapse
-
#call ⇒ Object
Executes the command.
-
#initialize(form, proposal) ⇒ UpdateProposal
constructor
Public: Initializes the command.
Constructor Details
#initialize(form, proposal) ⇒ UpdateProposal
Public: Initializes the command.
form - A form object with the params. proposal - the proposal to update.
14 15 16 17 18 |
# File 'app/commands/decidim/proposals/admin/update_proposal.rb', line 14 def initialize(form, proposal) @form = form @proposal = proposal @attached_to = proposal end |
Instance Method Details
#call ⇒ Object
Executes the command. Broadcasts these events:
-
:ok when everything is valid, together with the proposal.
-
:invalid if the form was not valid and we could not proceed.
Returns nothing.
26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 |
# File 'app/commands/decidim/proposals/admin/update_proposal.rb', line 26 def call return broadcast(:invalid) if form.invalid? (form.) if if return broadcast(:invalid) if end transaction do update_proposal document_cleanup!(include_all_attachments: true) (first_weight: ) if end broadcast(:ok, proposal) end |