Class: Decidim::Proposals::AdminLog::ProposalPresenter
- Inherits:
-
Log::BasePresenter
- Object
- Log::BasePresenter
- Decidim::Proposals::AdminLog::ProposalPresenter
- Defined in:
- app/presenters/decidim/proposals/admin_log/proposal_presenter.rb
Overview
This class holds the logic to present a ‘Decidim::Proposals::Proposal` for the `AdminLog` log.
Usage should be automatic and you should not need to call this class directly, but here is an example:
action_log = Decidim::ActionLog.last
view_helpers # => this comes from the views
ProposalPresenter.new(action_log, view_helpers).present
Instance Method Summary collapse
-
#initialize(action_log, view_helpers) ⇒ ProposalPresenter
constructor
A new instance of ProposalPresenter.
Constructor Details
#initialize(action_log, view_helpers) ⇒ ProposalPresenter
Returns a new instance of ProposalPresenter.
16 17 18 19 |
# File 'app/presenters/decidim/proposals/admin_log/proposal_presenter.rb', line 16 def initialize(action_log, view_helpers) super @proposal = action_log.resource if action_log.resource end |