Module: Decidim::Proposals::ApplicationHelper
- Includes:
- CheckBoxesTreeHelper, Comments::CommentsHelper, FollowableHelper, LikeableHelper, MapHelper, ControlVersionHelper, MapHelper, ProposalVotesHelper, RichTextEditorHelper, PaginateHelper
- Included in:
- ProposalCellsHelper, ProposalGCell, ProposalHistoryCell, ProposalMetadataCell
- Defined in:
- app/helpers/decidim/proposals/application_helper.rb
Overview
Custom helpers, scoped to the proposals engine.
Instance Method Summary collapse
-
#activity_filter_values ⇒ Object
Options to filter Proposals by activity.
- #component_name ⇒ Object
-
#filter_origin_values ⇒ Object
Explicitly commenting the used I18n keys so their are not flagged as unused i18n-tasks-use t(‘decidim.proposals.application_helper.filter_origin_values.official’) i18n-tasks-use t(‘decidim.proposals.application_helper.filter_origin_values.participants’) i18n-tasks-use t(‘decidim.proposals.application_helper.filter_origin_values.official’) i18n-tasks-use t(‘decidim.proposals.application_helper.filter_origin_values.meetings’) i18n-tasks-use t(‘decidim.proposals.application_helper.filter_origin_values.all’).
- #filter_proposals_state_values ⇒ Object
-
#filter_sections ⇒ Object
rubocop:disable Metrics/CyclomaticComplexity.
- #filter_type_values ⇒ Object
-
#humanize_proposal_state(state) ⇒ Object
Public: The state of a proposal in a way a human can understand.
- #layout_item_classes ⇒ Object
- #not_from_participatory_text(proposal) ⇒ Object
- #proposal_limit ⇒ Object
- #proposal_limit_enabled? ⇒ Boolean
-
#proposal_state_css_class(proposal) ⇒ Object
Public: The css class applied based on the proposal state.
- #proposal_state_css_style(proposal) ⇒ Object
-
#render_proposal_body(proposal) ⇒ Object
If the content is safe, HTML tags are sanitized, otherwise, they are stripped.
-
#safe_content? ⇒ Boolean
If the proposal is official or the rich text editor is enabled on the frontend, the proposal body is considered as safe content; that is unless safe_content_admin? is used and the proposal comes from a participatory text.
-
#safe_content_admin? ⇒ Boolean
For admin entered content, the proposal body can contain certain extra tags, such as iframes.
- #show_voting_rules? ⇒ Boolean
- #templates_available? ⇒ Boolean
-
#text_editor_for_proposal_body(form) ⇒ Object
Returns :text_area or :editor based on the organization’ settings.
Methods included from ControlVersionHelper
Methods included from MapHelper
#has_position?, #proposal_preview_data_for_map
Methods included from ProposalVotesHelper
#can_accumulate_votes_beyond_threshold?, #current_user_can_vote?, #minimum_votes_per_user_enabled?, #proposal_voted_by_user?, #remaining_minimum_votes_count_for_user, #remaining_votes_count_for_user, #threshold_per_proposal, #threshold_per_proposal_enabled?, #vote_limit_enabled?, #votes_blocked?, #votes_enabled?
Instance Method Details
#activity_filter_values ⇒ Object
Options to filter Proposals by activity.
130 131 132 133 134 135 136 137 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 130 def activity_filter_values base = [ ["all", t("decidim.proposals.proposals.filters.all")], ["my_proposals", t("decidim.proposals.proposals.filters.my_proposals")] ] base += [["voted", t("decidim.proposals.proposals.filters.voted")]] if current_settings.votes_enabled? base end |
#component_name ⇒ Object
223 224 225 226 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 223 def component_name i18n_key = "decidim.components.proposals.name" (defined?(current_component) && translated_attribute(current_component&.name).presence) || t(i18n_key) end |
#filter_origin_values ⇒ Object
Explicitly commenting the used I18n keys so their are not flagged as unused i18n-tasks-use t(‘decidim.proposals.application_helper.filter_origin_values.official’) i18n-tasks-use t(‘decidim.proposals.application_helper.filter_origin_values.participants’) i18n-tasks-use t(‘decidim.proposals.application_helper.filter_origin_values.official’) i18n-tasks-use t(‘decidim.proposals.application_helper.filter_origin_values.meetings’) i18n-tasks-use t(‘decidim.proposals.application_helper.filter_origin_values.all’)
145 146 147 148 149 150 151 152 153 154 155 156 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 145 def filter_origin_values scope = "decidim.proposals.application_helper.filter_origin_values" origin_values = [] origin_values << TreePoint.new("official", t("official", scope:)) if component_settings.official_proposals_enabled origin_values << TreePoint.new("participants", t("participants", scope:)) origin_values << TreePoint.new("meeting", t("meetings", scope:)) TreeNode.new( TreePoint.new("", t("all", scope:)), origin_values ) end |
#filter_proposals_state_values ⇒ Object
158 159 160 161 162 163 164 165 166 167 168 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 158 def filter_proposals_state_values Decidim::CheckBoxesTreeHelper::TreeNode.new( Decidim::CheckBoxesTreeHelper::TreePoint.new("", t("decidim.proposals.application_helper.filter_state_values.all")), [ Decidim::CheckBoxesTreeHelper::TreePoint.new("state_not_published", t("decidim.proposals.application_helper.filter_state_values.not_answered")) ] + Decidim::Proposals::ProposalState.where(component: current_component).where.not(token: "not_answered").map do |state| Decidim::CheckBoxesTreeHelper::TreePoint.new(state.token, translated_attribute(state.title)) end ) end |
#filter_sections ⇒ Object
rubocop:disable Metrics/CyclomaticComplexity
171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 171 def filter_sections @filter_sections ||= begin items = [] if component_settings.proposal_answering_enabled && current_settings.proposal_answering_enabled items.append(method: :with_any_state, name: "[with_any_state]", collection: filter_proposals_state_values, label: t("decidim.proposals.proposals.filters.state"), id: "state") end current_component.available_taxonomy_filters.each do |taxonomy_filter| items.append(method: :with_any_taxonomies, name: "[with_any_taxonomies][#{taxonomy_filter.root_taxonomy_id}]", collection: filter_taxonomy_values_for(taxonomy_filter), label: decidim_sanitize_translated(taxonomy_filter.name), id: "taxonomy-#{taxonomy_filter.root_taxonomy_id}") end if component_settings.official_proposals_enabled items.append(method: :with_any_origin, name: "[with_any_origin]", collection: filter_origin_values, label: t("decidim.proposals.proposals.filters.origin"), id: "origin") end if current_user items.append(method: :activity, name: "[activity]", collection: activity_filter_values, label: t("decidim.proposals.proposals.filters.activity"), id: "activity", type: :radio_buttons) end if @proposals.only_emendations.any? items.append(method: :type, name: "[type]", collection: filter_type_values, label: t("decidim.proposals.proposals.filters.amendment_type"), id: "amendment_type", type: :radio_buttons) end if linked_classes_for(Decidim::Proposals::Proposal).any? items.append( method: :related_to, name: "[related_to]", collection: linked_classes_filter_values_for(Decidim::Proposals::Proposal), label: t("decidim.proposals.proposals.filters.related_to"), id: "related_to", type: :radio_buttons ) end end # rubocop:enable Metrics/CyclomaticComplexity items.reject { |item| item[:collection].blank? } end |
#filter_type_values ⇒ Object
121 122 123 124 125 126 127 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 121 def filter_type_values [ ["all", t("decidim.proposals.application_helper.filter_type_values.all")], ["proposals", t("decidim.proposals.application_helper.filter_type_values.proposals")], ["amendments", t("decidim.proposals.application_helper.filter_type_values.amendments")] ] end |
#humanize_proposal_state(state) ⇒ Object
Public: The state of a proposal in a way a human can understand.
state - The String state of the proposal.
Returns a String.
24 25 26 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 24 def humanize_proposal_state(state) I18n.t(state, scope: "decidim.proposals.answers", default: :not_answered) end |
#layout_item_classes ⇒ Object
103 104 105 106 107 108 109 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 103 def layout_item_classes if show_voting_rules? "layout-item lg:pt-4" else "layout-item" end end |
#not_from_participatory_text(proposal) ⇒ Object
60 61 62 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 60 def not_from_participatory_text(proposal) proposal.participatory_text_level.nil? end |
#proposal_limit ⇒ Object
97 98 99 100 101 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 97 def proposal_limit return if component_settings.proposal_limit.zero? component_settings.proposal_limit end |
#proposal_limit_enabled? ⇒ Boolean
56 57 58 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 56 def proposal_limit_enabled? proposal_limit.present? end |
#proposal_state_css_class(proposal) ⇒ Object
Public: The css class applied based on the proposal state.
proposal - The proposal to evaluate.
Returns a String.
40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 40 def proposal_state_css_class(proposal) return "alert" if proposal.withdrawn? return if proposal.state.blank? case proposal.state when "accepted" "success" when "rejected", "withdrawn" "alert" when "evaluating" "warning" else "info" end end |
#proposal_state_css_style(proposal) ⇒ Object
28 29 30 31 32 33 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 28 def proposal_state_css_style(proposal) return "" if proposal.emendation? return "" if proposal.withdrawn? proposal.proposal_state&.css_style end |
#render_proposal_body(proposal) ⇒ Object
If the content is safe, HTML tags are sanitized, otherwise, they are stripped.
78 79 80 81 82 83 84 85 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 78 def render_proposal_body(proposal) sanitized = render_sanitized_content(proposal, :body) if safe_content? Decidim::ContentProcessor.render_without_format(sanitized).html_safe else Decidim::ContentProcessor.render(sanitized) end end |
#safe_content? ⇒ Boolean
If the proposal is official or the rich text editor is enabled on the frontend, the proposal body is considered as safe content; that is unless safe_content_admin? is used and the proposal comes from a participatory text.
67 68 69 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 67 def safe_content? rich_text_editor_in_public_views? || safe_content_admin? end |
#safe_content_admin? ⇒ Boolean
For admin entered content, the proposal body can contain certain extra tags, such as iframes.
73 74 75 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 73 def safe_content_admin? (@proposal.official? || @proposal.official_meeting?) && not_from_participatory_text(@proposal) end |
#show_voting_rules? ⇒ Boolean
111 112 113 114 115 116 117 118 119 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 111 def show_voting_rules? return false if !votes_enabled? || votes_blocked? return true if vote_limit_enabled? return true if threshold_per_proposal_enabled? return true if proposal_limit_enabled? return true if can_accumulate_votes_beyond_threshold? return true if minimum_votes_per_user_enabled? end |
#templates_available? ⇒ Boolean
228 229 230 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 228 def templates_available? Decidim.module_installed?(:templates) && defined?(Decidim::Templates::Template) && Decidim::Templates::Template.exists?(templatable: current_component) end |
#text_editor_for_proposal_body(form) ⇒ Object
Returns :text_area or :editor based on the organization’ settings.
88 89 90 91 92 93 94 95 |
# File 'app/helpers/decidim/proposals/application_helper.rb', line 88 def text_editor_for_proposal_body(form) = { value: form_presenter.body(strip_tags: !current_organization.rich_text_editor_in_public_views).strip, data: { controller: "character-counter" } } text_editor_for(form, :body, ) end |