Class: Decidim::Proposals::Proposal

Inherits:
ApplicationRecord show all
Includes:
Amendable, Coauthorable, DownloadYourData, Endorsable, FilterableResource, Fingerprintable, Followable, HasAttachments, HasCategory, HasComponent, HasReference, Loggable, NewsletterParticipant, CommentableProposal, ParticipatoryTextSection, Valuatable, Randomable, Reportable, Resourceable, ScopableResource, Searchable, Traceable, TranslatableAttributes, TranslatableResource
Defined in:
app/models/decidim/proposals/proposal.rb

Overview

The data store for a Proposal in the Decidim::Proposals component.

Constant Summary collapse

POSSIBLE_STATES =
%w(not_answered evaluating accepted rejected withdrawn).freeze

Constants included from ParticipatoryTextSection

Decidim::Proposals::ParticipatoryTextSection::LEVELS

Class Method Summary collapse

Instance Method Summary collapse

Class Method Details

.download_your_data_images(user) ⇒ Object



421
422
423
# File 'app/models/decidim/proposals/proposal.rb', line 421

def self.download_your_data_images(user)
  user_collection(user).map { |p| p.attachments.collect(&:file) }
end

.export_serializerObject



417
418
419
# File 'app/models/decidim/proposals/proposal.rb', line 417

def self.export_serializer
  Decidim::Proposals::ProposalSerializer
end

.log_presenter_class_for(_log) ⇒ Object



127
128
129
# File 'app/models/decidim/proposals/proposal.rb', line 127

def self.log_presenter_class_for(_log)
  Decidim::Proposals::AdminLog::ProposalPresenter
end

.newsletter_participant_ids(component) ⇒ Object



150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
# File 'app/models/decidim/proposals/proposal.rb', line 150

def self.newsletter_participant_ids(component)
  proposals = retrieve_proposals_for(component).uniq

  coauthors_recipients_ids = proposals.map { |p| p.notifiable_identities.pluck(:id) }.flatten.compact.uniq

  participants_has_voted_ids = Decidim::Proposals::ProposalVote.joins(:proposal).where(proposal: proposals).joins(:author).map(&:decidim_author_id).flatten.compact.uniq

  endorsements_participants_ids = Decidim::Endorsement.where(resource: proposals)
                                                      .where(decidim_author_type: "Decidim::UserBaseEntity")
                                                      .pluck(:decidim_author_id).to_a.compact.uniq

  commentators_ids = Decidim::Comments::Comment.user_commentators_ids_in(proposals)

  (endorsements_participants_ids + participants_has_voted_ids + coauthors_recipients_ids + commentators_ids).flatten.compact.uniq
end

.ransack(params = {}, options = {}) ⇒ Object



332
333
334
# File 'app/models/decidim/proposals/proposal.rb', line 332

def self.ransack(params = {}, options = {})
  ProposalSearch.new(self, params, options)
end

.ransackable_scopes(auth_object = nil) ⇒ Object



361
362
363
364
365
366
367
# File 'app/models/decidim/proposals/proposal.rb', line 361

def self.ransackable_scopes(auth_object = nil)
  base = [:with_any_origin, :with_any_state, :voted_by, :coauthored_by, :related_to, :with_any_scope, :with_any_category]
  return base unless auth_object&.admin?

  # Add extra scopes for admins for the admin panel searches
  base + [:valuator_role_ids_has]
end

.retrieve_proposals_for(component) ⇒ Object



141
142
143
144
145
146
147
148
# File 'app/models/decidim/proposals/proposal.rb', line 141

def self.retrieve_proposals_for(component)
  Decidim::Proposals::Proposal.where(component: component).joins(:coauthorships)
                              .includes(:votes, :endorsements)
                              .where(decidim_coauthorships: { decidim_author_type: "Decidim::UserBaseEntity" })
                              .not_hidden
                              .published
                              .except_withdrawn
end

.sort_by_translated_title_ascObject



386
387
388
389
# File 'app/models/decidim/proposals/proposal.rb', line 386

def self.sort_by_translated_title_asc
  field = Arel::Nodes::InfixOperation.new("->>", arel_table[:title], Arel::Nodes.build_quoted(I18n.locale))
  order(Arel::Nodes::InfixOperation.new("", field, Arel.sql("ASC")))
end

.sort_by_translated_title_descObject



391
392
393
394
# File 'app/models/decidim/proposals/proposal.rb', line 391

def self.sort_by_translated_title_desc
  field = Arel::Nodes::InfixOperation.new("->>", arel_table[:title], Arel::Nodes.build_quoted(I18n.locale))
  order(Arel::Nodes::InfixOperation.new("", field, Arel.sql("DESC")))
end

.sort_by_valuation_assignments_count_nulls_last_queryObject

Defines the base query so that ransack can actually sort by this value



337
338
339
340
341
342
343
344
345
346
# File 'app/models/decidim/proposals/proposal.rb', line 337

def self.sort_by_valuation_assignments_count_nulls_last_query
  <<-SQL.squish
  (
    SELECT COUNT(decidim_proposals_valuation_assignments.id)
    FROM decidim_proposals_valuation_assignments
    WHERE decidim_proposals_valuation_assignments.decidim_proposal_id = decidim_proposals_proposals.id
    GROUP BY decidim_proposals_valuation_assignments.decidim_proposal_id
  )
  SQL
end

.user_collection(author) ⇒ Object

Returns a collection scoped by an author. Overrides this method in DownloadYourData to support Coauthorable.



133
134
135
136
137
138
139
# File 'app/models/decidim/proposals/proposal.rb', line 133

def self.user_collection(author)
  return unless author.is_a?(Decidim::User)

  joins(:coauthorships)
    .where(decidim_coauthorships: { coauthorable_type: name })
    .where("decidim_coauthorships.decidim_author_id = ? AND decidim_coauthorships.decidim_author_type = ? ", author.id, author.class.base_class.name)
end

.valuator_role_ids_has(value) ⇒ Object

method to filter by assigned valuator role ID



349
350
351
352
353
354
355
356
357
358
359
# File 'app/models/decidim/proposals/proposal.rb', line 349

def self.valuator_role_ids_has(value)
  query = <<-SQL.squish
  :value = any(
    (SELECT decidim_proposals_valuation_assignments.valuator_role_id
    FROM decidim_proposals_valuation_assignments
    WHERE decidim_proposals_valuation_assignments.decidim_proposal_id = decidim_proposals_proposals.id
    )
  )
  SQL
  where(query, value: value)
end

.with_valuation_assigned_to(user, space) ⇒ Object



108
109
110
111
112
113
# File 'app/models/decidim/proposals/proposal.rb', line 108

def self.with_valuation_assigned_to(user, space)
  valuator_roles = space.user_roles(:valuator).where(user: user)

  includes(:valuation_assignments)
    .where(decidim_proposals_valuation_assignments: { valuator_role_id: valuator_roles })
end

Instance Method Details

#accepted?Boolean

Public: Checks if the organization has accepted a proposal.

Returns Boolean.

Returns:

  • (Boolean)


235
236
237
# File 'app/models/decidim/proposals/proposal.rb', line 235

def accepted?
  state == "accepted"
end

#allow_resource_permissions?Boolean

Public: Overrides the `allow_resource_permissions?` Resourceable concern method.

Returns:

  • (Boolean)


426
427
428
# File 'app/models/decidim/proposals/proposal.rb', line 426

def allow_resource_permissions?
  component.settings.resources_permissions_enabled
end

#answered?Boolean

Public: Checks if the organization has given an answer for the proposal.

Returns Boolean.

Returns:

  • (Boolean)


221
222
223
# File 'app/models/decidim/proposals/proposal.rb', line 221

def answered?
  answered_at.present?
end

#can_accumulate_supports_beyond_thresholdObject

Public: Can accumulate more votres than maximum for this proposal.

Returns true if can accumulate, false otherwise



307
308
309
# File 'app/models/decidim/proposals/proposal.rb', line 307

def can_accumulate_supports_beyond_threshold
  component.settings.can_accumulate_supports_beyond_threshold
end

#draft?Boolean

Public: Whether the proposal is a draft or not.

Returns:

  • (Boolean)


328
329
330
# File 'app/models/decidim/proposals/proposal.rb', line 328

def draft?
  published_at.nil?
end

#editable_by?(user) ⇒ Boolean

Checks whether the user can edit the given proposal.

user - the user to check for authorship

Returns:

  • (Boolean)


314
315
316
317
318
# File 'app/models/decidim/proposals/proposal.rb', line 314

def editable_by?(user)
  return true if draft? && created_by?(user)

  !published_state? && within_edit_time_limit? && !copied_from_other_component? && created_by?(user)
end

#evaluating?Boolean

Public: Checks if the organization has marked the proposal as evaluating it.

Returns Boolean.

Returns:

  • (Boolean)


249
250
251
# File 'app/models/decidim/proposals/proposal.rb', line 249

def evaluating?
  state == "evaluating"
end

#internal_stateObject

Public: Returns the internal state of the proposal.

Returns Boolean.



205
206
207
208
209
# File 'app/models/decidim/proposals/proposal.rb', line 205

def internal_state
  return amendment.state if emendation?

  self[:state]
end

#maximum_votesObject

Public: The maximum amount of votes allowed for this proposal.

Returns an Integer with the maximum amount of votes, nil otherwise.



288
289
290
291
292
293
# File 'app/models/decidim/proposals/proposal.rb', line 288

def maximum_votes
  maximum_votes = component.settings.threshold_per_proposal
  return nil if maximum_votes.zero?

  maximum_votes
end

#maximum_votes_reached?Boolean

Public: The maximum amount of votes allowed for this proposal. 0 means infinite.

Returns true if reached, false otherwise.

Returns:

  • (Boolean)


298
299
300
301
302
# File 'app/models/decidim/proposals/proposal.rb', line 298

def maximum_votes_reached?
  return false unless maximum_votes

  votes.count >= maximum_votes
end

#official?Boolean

Public: Whether the proposal is official or not.

Returns:

  • (Boolean)


276
277
278
# File 'app/models/decidim/proposals/proposal.rb', line 276

def official?
  authors.first.is_a?(Decidim::Organization)
end

#official_meeting?Boolean

Public: Whether the proposal is created in a meeting or not.

Returns:

  • (Boolean)


281
282
283
# File 'app/models/decidim/proposals/proposal.rb', line 281

def official_meeting?
  authors.first.instance_of?(Decidim::Meetings::Meeting)
end

#presenterObject

Returns the presenter for this author, to be used in the views. Required by ResourceRenderer.



260
261
262
# File 'app/models/decidim/proposals/proposal.rb', line 260

def presenter
  Decidim::Proposals::ProposalPresenter.new(self)
end

#process_amendment_state_change!Object



439
440
441
442
443
444
445
446
447
448
# File 'app/models/decidim/proposals/proposal.rb', line 439

def process_amendment_state_change!
  return unless %w(accepted rejected evaluating withdrawn).member?(amendment.state)

  PaperTrail.request(enabled: false) do
    update!(
      state: amendment.state,
      state_published_at: Time.current
    )
  end
end

#published?Boolean

Public: Checks if the proposal has been published or not.

Returns Boolean.

Returns:

  • (Boolean)


185
186
187
# File 'app/models/decidim/proposals/proposal.rb', line 185

def published?
  published_at.present?
end

#published_state?Boolean

Public: Checks if the organization has published the state for the proposal.

Returns Boolean.

Returns:

  • (Boolean)


214
215
216
# File 'app/models/decidim/proposals/proposal.rb', line 214

def published_state?
  emendation? || state_published_at.present?
end

#rejected?Boolean

Public: Checks if the organization has rejected a proposal.

Returns Boolean.

Returns:

  • (Boolean)


242
243
244
# File 'app/models/decidim/proposals/proposal.rb', line 242

def rejected?
  state == "rejected"
end

#reported_attributesObject

Public: Overrides the `reported_attributes` Reportable concern method.



265
266
267
# File 'app/models/decidim/proposals/proposal.rb', line 265

def reported_attributes
  [:title, :body]
end

#reported_content_urlObject

Public: Overrides the `reported_content_url` Reportable concern method.



254
255
256
# File 'app/models/decidim/proposals/proposal.rb', line 254

def reported_content_url
  ResourceLocatorPresenter.new(self).url
end

#reported_searchable_content_extrasObject

Public: Overrides the `reported_searchable_content_extras` Reportable concern method. Returns authors name or title in case it's a meeting



271
272
273
# File 'app/models/decidim/proposals/proposal.rb', line 271

def reported_searchable_content_extras
  [authors.map { |p| p.respond_to?(:name) ? p.name : p.title }.join("\n")]
end

#stateObject

Public: Returns the published state of the proposal.

Returns Boolean.



192
193
194
195
196
197
# File 'app/models/decidim/proposals/proposal.rb', line 192

def state
  return amendment.state if emendation?
  return nil unless published_state? || withdrawn?

  super
end

#update_votes_countObject

Public: Updates the vote count of this proposal.

Returns nothing. rubocop:disable Rails/SkipsModelValidations



170
171
172
# File 'app/models/decidim/proposals/proposal.rb', line 170

def update_votes_count
  update_columns(proposal_votes_count: votes.count)
end

#voted_by?(user) ⇒ Boolean

Public: Check if the user has voted the proposal.

Returns Boolean.

Returns:

  • (Boolean)


178
179
180
# File 'app/models/decidim/proposals/proposal.rb', line 178

def voted_by?(user)
  ProposalVote.where(proposal: self, author: user).any?
end

#withdrawable_by?(user) ⇒ Boolean

Checks whether the user can withdraw the given proposal.

user - the user to check for withdrawability.

Returns:

  • (Boolean)


323
324
325
# File 'app/models/decidim/proposals/proposal.rb', line 323

def withdrawable_by?(user)
  user && !withdrawn? && authored_by?(user) && !copied_from_other_component?
end

#withdrawn?Boolean

Public: Checks if the author has withdrawn the proposal.

Returns Boolean.

Returns:

  • (Boolean)


228
229
230
# File 'app/models/decidim/proposals/proposal.rb', line 228

def withdrawn?
  internal_state == "withdrawn"
end

#within_edit_time_limit?Boolean

Checks whether the proposal is inside the time window to be editable or not once published.

Returns:

  • (Boolean)


431
432
433
434
435
436
437
# File 'app/models/decidim/proposals/proposal.rb', line 431

def within_edit_time_limit?
  return true if draft?
  return true if component.settings.proposal_edit_time == "infinite"

  limit = updated_at + component.settings.proposal_edit_before_minutes.minutes
  Time.current < limit
end